/* Sistemas Integrados TEKNISA
   Gesto Consolidada

Nome: 9435_CVISTPO_ETL_CONT_GERENCIAL.sql

Descrio:
Procedure para preencher os fatos e dimenses da tabela ETLSALDCCUS 
a partir dos dados apurados do Saldo Contbil.

Criao:
16/12/2008 Leonardo Bispo

*/

IF EXISTS (SELECT * FROM SYSOBJECTS O, SYSUSERS U
                    WHERE
	        O.NAME = 'ETL_CONT_GERENCIAL' AND
	        TYPE = 'P' 	            AND
	        U.UID = O.UID               AND
	        U.NAME = USER)
	DROP PROCEDURE ETL_CONT_GERENCIAL
GO

CREATE PROCEDURE ETL_CONT_GERENCIAL @P_DTINI  DATETIME,
				    @P_DTFINA DATETIME
AS
   DECLARE @V_DTFIM       DATETIME;
   DECLARE @V_DTINI       DATETIME;
   DECLARE @V_NRSEQFATCUS VARCHAR(12);
   DECLARE @V_DTMES       VARCHAR(07);
   DECLARE @V_DTANO       VARCHAR(04);
   DECLARE @V_BIM         VARCHAR(6);
   DECLARE @V_TRI         VARCHAR(6);
   DECLARE @V_SEM         VARCHAR(6);
                          
   DECLARE @C_EMPRESA     VARCHAR(6);

   DECLARE @C_CDEMPRESA   VARCHAR(2);
   DECLARE @C_CDFILIAL    VARCHAR(4);
   DECLARE @C_CDCENTCUST  VARCHAR(4);
   DECLARE @C_CDCONTCTBL  VARCHAR(20);
   DECLARE @C_VRSALDOCCUS  DOUBLE PRECISION;


BEGIN
   EXEC NIVEL_ANT_CONTA;

   
   SET @V_DTINI  = @P_DTINI;

   SET @V_DTMES  =  (CONVERT (VARCHAR, MONTH (@P_DTINI))); 
   SET @V_DTANO  =  (CONVERT (VARCHAR, YEAR  (@P_DTINI)));    

   BEGIN TRANSACTION
      DELETE FROM ETLSALDCCUS
      WHERE (CDANOBI > @V_DTANO) OR
            (CDANOBI = @V_DTANO
             AND  CDMESBI >= @V_DTMES);
   COMMIT TRANSACTION;


    SELECT @V_NRSEQFATCUS = ISNULL(MAX(RIGHT(REPLICATE(0, 12) + NRSEQFATCUS, 12)),0)
    FROM   ETLSALDCCUS;


    DECLARE C_ATUCONTA CURSOR FOR
        (SELECT C.CDEMPRESA
         FROM EMPRESA E, CONTOPECTBL C
         WHERE C.CDEMPRESA = E.CDEMPRESA)
    OPEN C_ATUCONTA;                                    
    FETCH NEXT FROM C_ATUCONTA INTO 
                               @C_EMPRESA;
    
    WHILE (@@FETCH_STATUS = 0) BEGIN
        EXEC ATUALIZA_SALDO_GERENCIAL @C_EMPRESA;
    END;

    WHILE (@V_DTINI <= @P_DTFINA)BEGIN            
    
       SET @V_DTFIM = DATEADD(MM,1,@V_DTINI)-DAY(@V_DTINI); 
              
       DECLARE C_ETL CURSOR FOR
          (SELECT 
              CDEMPRESA,  CDFILIAL,   CDCENTCUST,
              CDCONTCTBL, SUM(VRSALDCCUS) AS VRSALDOCCUS
           FROM
              SALDCCUS
           WHERE 
              (DTSALDCCUS  BETWEEN @V_DTINI AND @V_DTFIM)
           GROUP BY
              CDEMPRESA,  CDFILIAL,   CDCENTCUST,
              CDCONTCTBL
          )
       OPEN C_ATUCONTA;                                    
       FETCH NEXT FROM C_ATUCONTA INTO 
                               @C_CDEMPRESA,  @C_CDFILIAL,   @C_CDCENTCUST,
              		       @C_CDCONTCTBL, @C_VRSALDOCCUS;
     
       WHILE (@@FETCH_STATUS = 0) BEGIN

             SET @V_DTMES  =  (CONVERT (VARCHAR, MONTH(@P_DTINI)));
             SET @V_DTANO  =  (CONVERT (VARCHAR, YEAR(@P_DTINI)));

             SET   @V_BIM  =  CONVERT(VARCHAR, (ROUND(((@V_DTMES - 1) / 2),0) + 1))+' Bim';
             SET   @V_TRI  =  CONVERT(VARCHAR, (ROUND(((@V_DTMES - 1) / 3),0) + 1))+' Tri';
             SET   @V_SEM  =  CONVERT(VARCHAR, (ROUND(((@V_DTMES - 1) / 6),0) + 1))+' Sem';

             SET @V_NRSEQFATCUS = ISNULL(MAX(RIGHT(REPLICATE(0, 12) + @V_NRSEQFATCUS, 12)),0);
              INSERT INTO ETLSALDCCUS
                      (NRSEQFATCUS, CDEMPRESA , CDFILIAL,
                       CDCENTCUST , CDCONTCTBL, CDMESBI,
                       BIMFATCUS  , TRIFATCUS , SEMFATCUS,
                       CDANOBI  , VRSALDCCUS)
              VALUES               
                       (@V_NRSEQFATCUS,    @C_CDEMPRESA,  @C_CDFILIAL,
                       @C_CDCENTCUST, @C_CDCONTCTBL, @V_DTMES,
                       @V_BIM+' Bim',   @V_TRI+' Tri',   @V_SEM+' Sem', 
                       @V_DTANO, @C_VRSALDOCCUS);
       END;

    COMMIT;
    SET @V_DTINI = DATEADD(MM, 1, @V_DTINI);
    END;
END;
GO

/*REGISTRANDO A PROCEDURE PARA O AGENDADOR DE EVENTOS*/
DECLARE @V_CDSEQUENCIAL VARCHAR(20);
BEGIN
   SELECT @V_CDSEQUENCIAL = COUNT(*)
   FROM   EVENTOAEV 
   WHERE  NMOBJBDEVEN = 'ETL_CONT_GERENCIAL';
    
   IF @V_CDSEQUENCIAL = 0 BEGIN   --> S ADICIONA A PROCEDURE SE ELA NO EXISTIR AINDA NO CADASTRO
      BEGIN TRAN;

         SELECT @V_CDSEQUENCIAL = RIGHT(REPLICATE('0', 10) + ISNULL(MAX(CDEVENTOAEV)+1,0), 10)
         FROM   EVENTOAEV;
        
         INSERT INTO EVENTOAEV 
            (CDEVENTOAEV, NMEVENTOAEV, IDTPEVENAEV, CDOPERINCL, DTINCLEVEN, NMOBJBDEVEN)
         VALUES
            (@V_CDSEQUENCIAL,'ETL Contbil Gerencial','STP',NULL,GETDATE(),'ETL_CONT_GERENCIAL');

         INSERT INTO PARAMEVENTO
            (CDEVENTOAEV,NMPARAMEVEN,IDDADPARAMEVEN,IDTIPPARAMEVEN,DSPARAMEVEN,IDPARAMNULL,VRTAMPARAM,NRSEQPARAMEVEN) 
         VALUES
            (@V_CDSEQUENCIAL,'P_DTINI','D','E','Data Inicial','N',0,'01');

         INSERT INTO PARAMEVENTO
            (CDEVENTOAEV,NMPARAMEVEN,IDDADPARAMEVEN,IDTIPPARAMEVEN,DSPARAMEVEN,IDPARAMNULL,VRTAMPARAM,NRSEQPARAMEVEN) 
         VALUES
            (@V_CDSEQUENCIAL,'P_DTFINA','D','E','Data Final','N',0,'02');

      COMMIT TRAN;

      IF @@ERROR <> 0 BEGIN
         ROLLBACK TRAN;
         PRINT 'Falha no cadastro de eventos agendveis.';
      END;
   END;
END;

GO

/*REGISTRANDO FATOS E DIMENSES*/
BEGIN                                           
   EXEC CRIA_FATOS 'ETLSALDCCUS', 'VRSALDCCUS',   'Saldo Gerencial',  'Contbil Gerencial';  
   
   EXEC CRIA_DIME 'ETLSALDCCUS',  'CDFILIAL',   'Filial',          'FILIAL',   'CDFILIAL',   'NMFILIAL',   NULL; 
   EXEC CRIA_DIME 'ETLSALDCCUS',  'CDEMPRESA',  'Empresa',         'EMPRESA',  'CDEMPRESA',  'NMEMPRESA',  NULL;    
   EXEC CRIA_DIME 'ETLSALDCCUS',  'CDCENTCUST', 'Centro de Custo', 'CENTCUST', 'CDCENTCUST', 'NMCENTCUST', NULL;
   EXEC CRIA_DIME 'ETLSALDCCUS',  'CDCONTCTBL', 'Conta Contbil',  'CONTCTBL', 'CDCONTCTBL', 'NMCONTCTBL', 'CDCONTANTE';
   EXEC CRIA_DIME 'ETLSALDCCUS', 'CDANOBI',   'Ano',             'ETLANOBI',       'CDANOBI',         'NMANOBI',         NULL;
   EXEC CRIA_DIME 'ETLSALDCCUS', 'SEMFATCUS',   'Semestre',  	   NULL,       NULL,         NULL,         NULL;
   EXEC CRIA_DIME 'ETLSALDCCUS', 'TRIFATCUS',   'Trimestre', 	   NULL,       NULL,         NULL,         NULL;
   EXEC CRIA_DIME 'ETLSALDCCUS', 'BIMFATCUS',   'Bimestre',  	   NULL,       NULL,         NULL,         NULL;
   EXEC CRIA_DIME 'ETLSALDCCUS', 'CDMESBI',   'Ms',      	   'ETLMESBI',       'CDMESBI',         'NMMESBI',         NULL;
END;

GO

-- VERSAO --
EXEC ATUVEROBJ 'ETL_CONT_GERENCIAL','5.05.020';
GO
